import warnings

import httpx
import prefect.runtime.flow_run as run
from marvin.beta.assistants import Assistant
from marvin.utilities.tools import custom_partial
from prefect import flow, task
from prefect.utilities.asyncutils import run_coro_as_sync
from rich.status import Status

from raggy.documents import Document
from raggy.loaders.github import GitHubRepoLoader
from raggy.vectorstores.tpuf import TurboPuffer, query_namespace

TPUF_NS = "demo"


@task(
    task_run_name="load documents from {repo}",
    cache_key_fn=lambda *_, **__: httpx.get(  # update embeddings on changes to the repo
        f"https://api.github.com/repos/{run.parameters['repo']}/commits/main"
    ).json()["sha"],
)
async def gather_documents(repo: str) -> list[Document]:
    return await GitHubRepoLoader(repo=repo).load()


@task
async def upsert_documents(documents: list[Document]):
    async with TurboPuffer(namespace=TPUF_NS) as tpuf:
        print(f"Upserting {len(documents)} documents into {TPUF_NS}")
        await tpuf.upsert(documents)


@flow(flow_run_name="{repo}")
async def ingest_repo(repo: str):
    """repo should be in the format 'owner/repo'"""
    documents = await gather_documents(repo)
    await upsert_documents(documents)


@flow(log_prints=True)
async def chat_with_repo(initial_message: str | None = None, clean_up: bool = True):
    try:
        with Assistant(
            model="gpt-4o",
            name="Raggy Expert",
            instructions=(
                "Use your tools to ingest and chat about a GitHub repo. "
                "Let the user choose questions, query as needed to get good answers. "
                "You must find documented syntax in the repo of question before suggesting it."
            ),
            tools=[
                ingest_repo,
                custom_partial(
                    task(task_run_name="Q: {query_text}")(query_namespace),
                    namespace=TPUF_NS,
                ),
            ],
        ) as assistant:
            assistant.chat(initial_message=initial_message)  # type: ignore

    finally:
        if clean_up:
            async with TurboPuffer(namespace=TPUF_NS) as tpuf:
                with Status(f"Cleaning up namespace {TPUF_NS}"):
                    await task(tpuf.reset)()


if __name__ == "__main__":
    warnings.filterwarnings("ignore", category=UserWarning)
    run_coro_as_sync(chat_with_repo("lets chat about zzstoatzz/prefect-bot"))



from datetime import timedelta
from typing import Literal

from bs4 import BeautifulSoup
from prefect import flow, task
from prefect.tasks import task_input_hash

import raggy
from raggy.documents import Document
from raggy.loaders.base import Loader
from raggy.loaders.github import GitHubRepoLoader
from raggy.loaders.web import SitemapLoader
from raggy.vectorstores.chroma import Chroma


def html_parser(html: str) -> str:
    import trafilatura

    trafilatura_config = trafilatura.settings.use_config()
    # disable signal, so it can run in a worker thread
    # https://github.com/adbar/trafilatura/issues/202
    trafilatura_config.set("DEFAULT", "EXTRACTION_TIMEOUT", "0")
    return (
        trafilatura.extract(html, config=trafilatura_config)
        or BeautifulSoup(html, "html.parser").get_text()
    )


raggy.settings.html_parser = html_parser

prefect_loaders = [
    SitemapLoader(
        urls=[
            "https://docs-3.prefect.io/sitemap.xml",
            "https://prefect.io/sitemap.xml",
        ],
        exclude=["api-ref", "www.prefect.io/events"],
    ),
    GitHubRepoLoader(
        repo="PrefectHQ/prefect",
        include_globs=["flows/"],
    ),
]


@task(
    retries=2,
    retry_delay_seconds=[3, 60],
    cache_key_fn=task_input_hash,
    cache_expiration=timedelta(days=1),
    task_run_name="Run {loader.__class__.__name__}",
    persist_result=True,
    # refresh_cache=True,
)
async def run_loader(loader: Loader) -> list[Document]:
    return await loader.load()


@flow(name="Update Knowledge", log_prints=True)
async def refresh_chroma(
    collection_name: str = "default",
    chroma_client_type: Literal["base", "http"] = "base",
    mode: Literal["upsert", "reset"] = "upsert",
):
    """Flow updating vectorstore with info from the Prefect community."""
    documents = [
        doc for future in run_loader.map(prefect_loaders) for doc in future.result()
    ]

    print(f"Loaded {len(documents)} documents from the Prefect community.")

    async with Chroma(
        collection_name=collection_name, client_type=chroma_client_type
    ) as chroma:
        if mode == "reset":
            await chroma.reset_collection()
            docs = await chroma.add(documents)
        elif mode == "upsert":
            docs = await chroma.upsert(documents)
        else:
            raise ValueError(f"Unknown mode: {mode!r} (expected 'upsert' or 'reset')")

        print(f"Added {len(docs)} documents to the {collection_name} collection.")


if __name__ == "__main__":
    import asyncio

    asyncio.run(
        refresh_chroma(
            collection_name="testing", chroma_client_type="base", mode="reset"
        )
    )



from datetime import timedelta

from bs4 import BeautifulSoup
from prefect import flow, task
from prefect.tasks import task_input_hash
from prefect.utilities.annotations import quote

import raggy
from raggy.documents import Document
from raggy.loaders.base import Loader
from raggy.loaders.github import GitHubRepoLoader
from raggy.loaders.web import SitemapLoader
from raggy.vectorstores.tpuf import TurboPuffer


def html_parser(html: str) -> str:
    import trafilatura

    trafilatura_config = trafilatura.settings.use_config()  # type: ignore
    # disable signal, so it can run in a worker thread
    # https://github.com/adbar/trafilatura/issues/202
    trafilatura_config.set("DEFAULT", "EXTRACTION_TIMEOUT", "0")
    return (
        trafilatura.extract(html, config=trafilatura_config)
        or BeautifulSoup(html, "html.parser").get_text()
    )


raggy.settings.html_parser = html_parser


prefect_loaders = [
    SitemapLoader(
        urls=[
            "https://docs-3.prefect.io/sitemap.xml",
            "https://prefect.io/sitemap.xml",
        ],
        exclude=["api-ref", "www.prefect.io/events"],
    ),
    GitHubRepoLoader(
        repo="PrefectHQ/prefect",
        include_globs=["flows/"],
    ),
]


@task(
    retries=2,
    retry_delay_seconds=[3, 60],
    cache_key_fn=task_input_hash,
    cache_expiration=timedelta(days=1),
    task_run_name="Run {loader.__class__.__name__}",
    persist_result=True,
    # refresh_cache=True,
)
async def run_loader(loader: Loader) -> list[Document]:
    return await loader.load()


@flow(name="Update Knowledge", log_prints=True)
async def refresh_tpuf_namespace(namespace: str = "testing", reset: bool = False):
    """Flow updating vectorstore with info from the Prefect community."""
    documents: list[Document] = [
        doc
        for future in run_loader.map(quote(prefect_loaders))  # type: ignore
        for doc in future.result()
    ]

    print(f"Loaded {len(documents)} documents from the Prefect community.")

    async with TurboPuffer(namespace=namespace) as tpuf:
        if reset:
            await tpuf.reset()
            print(f"RESETTING: Deleted all documents from tpuf ns {namespace!r}.")

        await tpuf.upsert(documents=documents)

    print(f"Updated tpuf ns {namespace!r} with {len(documents)} documents.")


if __name__ == "__main__":
    import asyncio

    asyncio.run(refresh_tpuf_namespace(namespace="prefect-3", reset=True))  # type: ignore



import argparse
import os
import sys
from datetime import datetime, timezone
from pathlib import Path

import openai
from prompt_toolkit import PromptSession
from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
from prompt_toolkit.history import FileHistory
from rich.console import Console, ConsoleOptions, RenderResult
from rich.live import Live
from rich.markdown import CodeBlock, Markdown
from rich.status import Status
from rich.syntax import Syntax
from rich.text import Text


class SimpleCodeBlock(CodeBlock):
    def __rich_console__(
        self, console: Console, options: ConsoleOptions
    ) -> RenderResult:
        code = str(self.text).rstrip()
        yield Text(self.lexer_name, style="dim")
        yield Syntax(
            code,
            self.lexer_name,
            theme=self.theme,
            background_color="default",
            word_wrap=True,
        )
        yield Text(f"/{self.lexer_name}", style="dim")


Markdown.elements["fence"] = SimpleCodeBlock


def app() -> int:
    parser = argparse.ArgumentParser(
        prog="aicli",
        description="""\
OpenAI powered AI CLI (thank you samuelcolvin)

Special prompts:
* `show-markdown` - show the markdown output from the previous response
* `multiline` - toggle multiline mode
""",
    )
    parser.add_argument(
        "prompt", nargs="?", help="AI Prompt, if omitted fall into interactive mode"
    )

    parser.add_argument(
        "--no-stream",
        action="store_true",
        help="Whether to stream responses from OpenAI",
    )

    parser.add_argument("--version", action="store_true", help="Show version and exit")

    args = parser.parse_args()

    console = Console()
    console.print("OpenAI powered AI CLI", style="green bold", highlight=False)
    if args.version:
        return 0

    try:
        openai_api_key = os.environ["OPENAI_API_KEY"]
    except KeyError:
        console.print(
            "You must set the OPENAI_API_KEY environment variable", style="red"
        )
        return 1

    client = openai.OpenAI(api_key=openai_api_key)

    now_utc = datetime.now(timezone.utc)
    t = now_utc.astimezone().tzinfo.tzname(now_utc)  # type: ignore
    setup = f"""\
Help the user by responding to their request, the output should 
be concise and always written in markdown. The current date and time
is {datetime.now()} {t}. The user is running {sys.platform}."""

    stream = not args.no_stream
    messages = [{"role": "system", "content": setup}]

    if args.prompt:
        messages.append({"role": "user", "content": args.prompt})
        try:
            ask_openai(client, messages, stream, console)
        except KeyboardInterrupt:
            pass
        return 0

    history = Path().home() / ".openai-prompt-history.txt"
    session = PromptSession(history=FileHistory(str(history)))
    multiline = False

    while True:
        try:
            text = session.prompt(
                "aicli ➤ ", auto_suggest=AutoSuggestFromHistory(), multiline=multiline
            )
        except (KeyboardInterrupt, EOFError):
            return 0

        if not text.strip():
            continue

        ident_prompt = text.lower().strip(" ").replace(" ", "-")
        if ident_prompt == "show-markdown":
            last_content = messages[-1]["content"]
            console.print("[dim]Last markdown output of last question:[/dim]\n")
            console.print(
                Syntax(last_content, lexer="markdown", background_color="default")
            )
            continue
        elif ident_prompt == "multiline":
            multiline = not multiline
            if multiline:
                console.print(
                    "Enabling multiline mode. "
                    "[dim]Press [Meta+Enter] or [Esc] followed by [Enter] to accept input.[/dim]"
                )
            else:
                console.print("Disabling multiline mode.")
            continue

        messages.append({"role": "user", "content": text})

        try:
            content = ask_openai(client, messages, stream, console)
        except KeyboardInterrupt:
            return 0
        messages.append({"role": "assistant", "content": content})


def ask_openai(
    client: openai.OpenAI,
    messages: list[dict[str, str]],
    stream: bool,
    console: Console,
) -> str:
    with Status("[dim]Working on it…[/dim]", console=console):
        response = client.chat.completions.create(
            model="gpt-4", messages=messages, stream=stream
        )

    console.print("\nResponse:", style="green")
    if stream:
        content = ""
        interrupted = False
        with Live("", refresh_per_second=15, console=console) as live:
            try:
                for chunk in response:
                    if chunk.choices[0].finish_reason is not None:
                        break
                    chunk_text = chunk.choices[0].delta.content
                    content += chunk_text
                    live.update(Markdown(content))
            except KeyboardInterrupt:
                interrupted = True

        if interrupted:
            console.print("[dim]Interrupted[/dim]")
    else:
        content = response.choices[0].message.content
        console.print(Markdown(content))

    return content


if __name__ == "__main__":
    sys.exit(app())
